@@ -18,11 +18,6 @@
 #' s(".mydiv") |>
 #'   elem_name()
 #'
-#' \dontshow{
-#' # Clean up all connections and invalidate default chromote object
-#' selenider_cleanup()
-#' }
-#'
 #' @family properties
 #'
 #' @export
@@ -71,11 +66,6 @@ element_name <- function(x, session, driver) {
 #' s("p") |>
 #'   elem_text()
 #'
-#' \dontshow{
-#' # Clean up all connections and invalidate default chromote object
-#' selenider_cleanup()
-#' }
-#'
 #' @export
 elem_text <- function(x, timeout = NULL) {
   check_number_decimal(timeout, allow_null = TRUE)
@@ -150,11 +140,6 @@ chromote_get_text <- function(x, driver) {
 #' s("input[type='number']") |>
 #'   elem_value(ptype = integer())
 #'
-#' \dontshow{
-#' # Clean up all connections and invalidate default chromote object
-#' selenider_cleanup()
-#' }
-#'
 #' @export
 elem_attr <- function(x, name, default = NULL, timeout = NULL) {
   check_string(name)
@@ -275,7 +260,39 @@ elem_value <- function(x, ptype = character(), timeout = NULL) {
 }
 
 element_value <- function(x, session, driver) {
-  execute_js_fn_on("x => x.value", x, session = session, driver = driver)
+  type <- element_input_type(x, session = session, driver = driver)
+
+  if (type == "select") {
+    element_select_value(x, session = session, driver = driver)
+  } else if (type == "contenteditable") {
+    element_text(x, session = session, driver = driver)
+  } else {
+    execute_js_fn_on("x => x.value", x, session = session, driver = driver)
+  }
+}
+
+element_select_value <- function(x, session, driver) {
+  result <- execute_js_fn_on("function(x) {
+    if (x.type == 'select-one') {
+      return x.options[x.selectedIndex].value;
+    } else {
+      let result = [];
+
+      for (let i = 0; i < x.options.length; i++) {
+        if (x.options[i].selected) {
+          result.push(x.options[i].value);
+        }
+      }
+
+      return result;
+    }
+  }", x, session = session, driver = driver)
+
+  if (is.list(result)) {
+    unlist(result)
+  } else {
+    result
+  }
 }
 
 convert_value <- function(x, ptype) {
@@ -317,11 +334,6 @@ convert_value <- function(x, ptype) {
 #' s("p") |>
 #'   elem_css_property("color")
 #'
-#' \dontshow{
-#' # Clean up all connections and invalidate default chromote object
-#' selenider_cleanup()
-#' }
-#'
 #' @export
 elem_css_property <- function(x, name, timeout = NULL) {
   check_class(x, "selenider_element")
@@ -411,11 +423,6 @@ chromote_get_css_property <- function(x, name, default, driver) {
 #' ss("div") |>
 #'   length()
 #'
-#' \dontshow{
-#' # Clean up all connections and invalidate default chromote object
-#' selenider_cleanup()
-#' }
-#'
 #' @export
 elem_size <- function(x, timeout = NULL) {
   timeout <- get_timeout(timeout, x$timeout)
